@@ -216,8 +216,8 @@ ephy_about_handler_handle_epiphany (EphyAboutHandler       *handler,
                           "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">"
                           "</head><body class=\"epiphany-body\">"
                           "<div id=\"ephytext\">"
-                          "“Il semble que la perfection soit atteinte non quand il n'y a plus rien à"
-                          " ajouter, mais quand il n'y a plus rien à retrancher.”"
+                          "« Il semble que la perfection soit atteinte non quand il n'y a plus rien à"
+                          " ajouter, mais quand il n'y a plus rien à retrancher. »"
                           "</div>"
                           "<div id=\"from\">"
                           "<!-- Terre des Hommes, III: L'Avion, p. 60 -->"
@@ -264,6 +264,9 @@ handle_applications_finished_cb (EphyAboutHandler       *handler,
     for (p = applications; p; p = p->next) {
       EphyWebApplication *app = (EphyWebApplication *)p->data;
 
+      if (ephy_web_application_is_system (app))
+        continue;
+
       g_string_append_printf (data_str,
                               "<tbody><tr id =\"%s\">"
                               "<td class=\"icon\"><img width=64 height=64 src=\"file://%s\"></img></td>"
@@ -485,7 +488,7 @@ ephy_about_handler_handle_incognito (EphyAboutHandler       *handler,
                           "<link href=\""EPHY_PAGE_TEMPLATE_ABOUT_CSS "\" rel=\"stylesheet\" type=\"text/css\">\n"
                           "</head>\n"
                           "<body class=\"incognito-body\">\n"
-                          "  <img class=\"incognito-body-image\" src=\"ephy-resource:///org/gnome/epiphany/private-mode.svg\">\n" \
+                          "  <img class=\"incognito-body-image\" src=\"ephy-resource:///org/gnome/epiphany/page-icons/private-mode.svg\">\n" \
                           "  <br/>\n"
                           "  <h1>%s</h1>\n"
                           "  <p>%s</p>\n"
