@@ -505,12 +505,12 @@ history_service_host_deleted_cb (EphyHistoryService *service,
                                  EphyEmbedShell     *shell)
 {
   EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
-  g_autoptr (SoupURI) deleted_uri = NULL;
+  g_autoptr (GUri) deleted_uri = NULL;
 
-  deleted_uri = soup_uri_new (deleted_url);
+  deleted_uri = g_uri_parse (deleted_url, G_URI_FLAGS_NONE, NULL);
   webkit_web_context_send_message_to_all_extensions (priv->web_context,
                                                      webkit_user_message_new ("History.DeleteHost",
-                                                                              g_variant_new ("s", soup_uri_get_host (deleted_uri))));
+                                                                              g_variant_new ("s", g_uri_get_host (deleted_uri))));
 }
 
 static void
@@ -638,12 +638,11 @@ ephy_embed_shell_get_global_gsb_service (EphyEmbedShell *shell)
   g_assert (EPHY_IS_EMBED_SHELL (shell));
 
   if (!priv->global_gsb_service) {
-    g_autofree char *api_key = NULL;
     g_autofree char *db_path = NULL;
+    g_autofree char *default_cache_dir = ephy_default_cache_dir ();
 
-    api_key = g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_GSB_API_KEY);
-    db_path = g_build_filename (ephy_default_cache_dir (), EPHY_GSB_FILE, NULL);
-    priv->global_gsb_service = ephy_gsb_service_new (api_key, db_path);
+    db_path = g_build_filename (default_cache_dir, EPHY_GSB_FILE, NULL);
+    priv->global_gsb_service = ephy_gsb_service_new (GSB_API_KEY, db_path);
   }
 
   return priv->global_gsb_service;
@@ -715,6 +714,8 @@ ephy_resource_request_cb (WebKitURISchemeRequest *request)
 {
   const char *path;
   gsize size;
+  WebKitWebView *request_view;
+  const char *uri;
   g_autoptr (GInputStream) stream = NULL;
   g_autoptr (GError) error = NULL;
 
@@ -724,11 +725,34 @@ ephy_resource_request_cb (WebKitURISchemeRequest *request)
     return;
   }
 
-  stream = g_resources_open_stream (path, 0, &error);
-  if (stream)
-    webkit_uri_scheme_request_finish (request, stream, size, NULL);
-  else
-    webkit_uri_scheme_request_finish_error (request, error);
+  request_view = webkit_uri_scheme_request_get_web_view (request);
+  uri = webkit_web_view_get_uri (request_view);
+
+  /* ephy-resource:// requests bypass CORS in order to allow ephy-pdf:// to
+   * access ephy-resource://. Accordingly, we need some custom security to
+   * prevent websites from directly accessing ephy-resource://.
+   *
+   * We'll have to leave open /page-icons and /page-templates since they are
+   * needed for our alternate HTML error pages.
+   */
+  if (g_str_has_prefix (uri, "ephy-resource:") ||
+      g_str_has_prefix (path, "/org/gnome/epiphany/page-icons/") ||
+      g_str_has_prefix (path, "/org/gnome/epiphany/page-templates/") ||
+      (g_str_has_prefix (uri, "ephy-pdf:") && g_str_has_prefix (path, "/org/gnome/epiphany/pdfjs/")) ||
+      (g_str_has_prefix (uri, "ephy-reader:") && g_str_has_prefix (path, "/org/gnome/epiphany/readability/")) ||
+      (g_str_has_prefix (uri, "ephy-source:") && g_str_has_prefix (path, "/org/gnome/epiphany/highlightjs/"))) {
+    stream = g_resources_open_stream (path, 0, &error);
+    if (stream)
+      webkit_uri_scheme_request_finish (request, stream, size, NULL);
+    else
+      webkit_uri_scheme_request_finish_error (request, error);
+    return;
+  }
+
+  error = g_error_new (WEBKIT_NETWORK_ERROR, WEBKIT_NETWORK_ERROR_FAILED,
+                       _("URI %s not authorized to access Epiphany resource %s"),
+                       uri, path);
+  webkit_uri_scheme_request_finish_error (request, error);
 }
 
 static void
@@ -863,6 +887,27 @@ enable_itp_setting_changed_cb (GSettings      *settings,
                                                                        EPHY_PREFS_WEB_ENABLE_ITP));
 }
 
+static void
+update_system_scrollbars (EphyEmbedShell *shell)
+{
+  EphyEmbedShellPrivate *priv = ephy_embed_shell_get_instance_private (shell);
+  const char *theme_name;
+  gboolean enabled;
+
+  g_object_get (gtk_settings_get_default (),
+                "gtk-theme_name", &theme_name,
+                NULL);
+
+  /* Don't enable system scrollbars for Adwaita */
+  enabled = g_strcmp0 (theme_name, "Adwaita") &&
+            g_strcmp0 (theme_name, "Adwaita-dark") &&
+            g_strcmp0 (theme_name, "HighContrast") &&
+            g_strcmp0 (theme_name, "HighContrastInverse");
+
+  webkit_web_context_set_use_system_appearance_for_scrollbars (priv->web_context,
+                                                               enabled);
+}
+
 static void
 ephy_embed_shell_startup (GApplication *application)
 {
@@ -902,9 +947,6 @@ ephy_embed_shell_startup (GApplication *application)
     /* Favicon Database */
     favicon_db_path = g_build_filename (ephy_cache_dir (), "icondatabase", NULL);
     webkit_web_context_set_favicon_database_directory (priv->web_context, favicon_db_path);
-
-    /* Do not ignore TLS errors. */
-    webkit_web_context_set_tls_errors_policy (priv->web_context, WEBKIT_TLS_ERRORS_POLICY_FAIL);
   }
 
   /* about: URIs handler */
@@ -958,6 +1000,13 @@ ephy_embed_shell_startup (GApplication *application)
 
   g_signal_connect_object (EPHY_SETTINGS_WEB, "changed::enable-itp",
                            G_CALLBACK (enable_itp_setting_changed_cb), shell, 0);
+
+  update_system_scrollbars (shell);
+
+  g_signal_connect_swapped (gtk_settings_get_default (),
+                            "notify::gtk-theme-name",
+                            G_CALLBACK (update_system_scrollbars),
+                            shell);
 }
 
 static void
