@@ -41,7 +41,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libsoup/soup.h>
 #include <string.h>
 #include <webkit2/webkit2.h>
 
@@ -57,10 +56,11 @@ struct _EphyLocationEntry {
   GtkOverlay parent_instance;
 
   GtkWidget *url_entry;
-  GtkWidget *bookmark;
-  GtkWidget *bookmark_event_box;
-  GtkWidget *reader_mode;
-  GtkWidget *reader_mode_event_box;
+  GtkWidget *page_action_box;
+  GtkWidget *bookmark_icon;
+  GtkWidget *bookmark_button;
+  GtkWidget *reader_mode_icon;
+  GtkWidget *reader_mode_button;
 
   GBinding *paste_binding;
 
@@ -87,6 +87,8 @@ struct _EphyLocationEntry {
   guint original_address : 1;
 
   EphySecurityLevel security_level;
+  EphyAdaptiveMode adaptive_mode;
+  EphyBookmarkIconState icon_state;
 };
 
 static gboolean ephy_location_entry_reset_internal (EphyLocationEntry *,
@@ -133,6 +135,71 @@ entry_button_release (GtkWidget *widget,
   return GDK_EVENT_STOP;
 }
 
+static void
+update_entry_style (EphyLocationEntry *self)
+{
+  PangoAttrList *attrs;
+  PangoAttribute *color_normal;
+  PangoAttribute *color_dimmed;
+  PangoAttribute *scaled;
+  g_autoptr (GUri) uri = NULL;
+  const char *text = gtk_entry_get_text (GTK_ENTRY (self->url_entry));
+  const char *host;
+  const char *base_domain;
+  char *sub_string;
+
+  attrs = pango_attr_list_new ();
+
+  if (self->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW) {
+    scaled = pango_attr_scale_new (PANGO_SCALE_SMALL);
+    pango_attr_list_insert (attrs, scaled);
+  }
+
+  if (gtk_widget_has_focus (self->url_entry))
+    goto out;
+
+  uri = g_uri_parse (text, G_URI_FLAGS_NONE, NULL);
+  if (!uri)
+    goto out;
+
+  host = g_uri_get_host (uri);
+  if (!host || strlen (host) == 0)
+    goto out;
+
+  base_domain = soup_tld_get_base_domain (host, NULL);
+  if (!base_domain)
+    goto out;
+
+  sub_string = strstr (text, base_domain);
+  if (!sub_string)
+    goto out;
+
+  /* Complete text is dimmed */
+  color_dimmed = pango_attr_foreground_alpha_new (32768);
+  pango_attr_list_insert (attrs, color_dimmed);
+
+  /* Base domain with normal style */
+  color_normal = pango_attr_foreground_alpha_new (65535);
+  color_normal->start_index = sub_string - text;
+  color_normal->end_index = color_normal->start_index + strlen (base_domain);
+  pango_attr_list_insert (attrs, color_normal);
+
+out:
+  gtk_entry_set_attributes (GTK_ENTRY (self->url_entry), attrs);
+  pango_attr_list_unref (attrs);
+}
+
+static gboolean
+entry_focus_in_event (GtkWidget *widget,
+                      GdkEvent  *event,
+                      gpointer   user_data)
+{
+  EphyLocationEntry *self = EPHY_LOCATION_ENTRY (user_data);
+
+  update_entry_style (self);
+  return GDK_EVENT_PROPAGATE;
+}
+
 static gboolean
 entry_focus_out_event (GtkWidget *widget,
                        GdkEvent  *event,
@@ -140,6 +207,8 @@ entry_focus_out_event (GtkWidget *widget,
 {
   EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
 
+  update_entry_style (entry);
+
   if (((GdkEventButton *)event)->button != GDK_BUTTON_PRIMARY)
     return GDK_EVENT_PROPAGATE;
 
@@ -194,6 +263,7 @@ ephy_location_entry_title_widget_set_address (EphyTitleWidget *widget,
   g_autofree char *effective_text = NULL;
   g_autofree char *selection = NULL;
   int start, end;
+  const char *final_text;
 
   g_assert (widget);
 
@@ -224,12 +294,15 @@ ephy_location_entry_title_widget_set_address (EphyTitleWidget *widget,
     text = "";
   }
 
+  final_text = effective_text ? effective_text : text;
+
   /* First record the new hash, then update the entry text */
-  entry->hash = g_str_hash (effective_text ? effective_text : text);
+  entry->hash = g_str_hash (final_text);
 
   entry->block_update = TRUE;
   g_signal_handlers_block_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
-  gtk_entry_set_text (GTK_ENTRY (entry->url_entry), effective_text ? effective_text : text);
+  gtk_entry_set_text (GTK_ENTRY (entry->url_entry), final_text);
+  update_entry_style (entry);
   g_signal_handlers_unblock_by_func (entry->url_entry, G_CALLBACK (editable_changed_cb), entry);
 
   dzl_suggestion_entry_hide_suggestions (DZL_SUGGESTION_ENTRY (entry->url_entry));
@@ -748,18 +821,11 @@ icon_button_icon_press_event_cb (GtkWidget            *widget,
   return FALSE;
 }
 
-static gboolean
-bookmark_icon_button_press_event_cb (GtkWidget         *entry,
-                                     GdkEventButton    *event,
-                                     EphyLocationEntry *lentry)
+static void
+bookmark_icon_button_clicked_cb (GtkButton         *button,
+                                 EphyLocationEntry *lentry)
 {
-  if (((event->type == GDK_BUTTON_PRESS &&
-        event->button == 1) ||
-       (event->type == GDK_TOUCH_BEGIN))) {
-    g_signal_emit (lentry, signals[BOOKMARK_CLICKED], 0);
-  }
-
-  return TRUE;
+  g_signal_emit (lentry, signals[BOOKMARK_CLICKED], 0);
 }
 
 static GtkBorder
@@ -982,7 +1048,7 @@ update_reader_icon (EphyLocationEntry *entry)
   else
     name = "ephy-reader-mode-symbolic";
 
-  gtk_image_set_from_icon_name (GTK_IMAGE (entry->reader_mode),
+  gtk_image_set_from_icon_name (GTK_IMAGE (entry->reader_mode_icon),
                                 name, GTK_ICON_SIZE_MENU);
 }
 
@@ -1033,42 +1099,42 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_container_add (GTK_CONTAINER (event), box);
   g_signal_connect (G_OBJECT (box), "size-allocate", G_CALLBACK (button_box_size_allocated_cb), entry);
-  gtk_widget_set_halign (box, GTK_ALIGN_END);
   gtk_widget_set_valign (box, GTK_ALIGN_CENTER);
+  gtk_widget_set_halign (box, GTK_ALIGN_END);
   gtk_widget_show (box);
 
+  /* Page action box */
+  entry->page_action_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_pack_start (GTK_BOX (box), entry->page_action_box, FALSE, FALSE, 0);
+
   context = gtk_widget_get_style_context (box);
   gtk_style_context_add_class (context, "entry_icon_box");
 
   /* Bookmark */
-  entry->bookmark_event_box = gtk_event_box_new ();
-  gtk_widget_set_tooltip_text (entry->bookmark_event_box, _("Bookmark this page"));
-  entry->bookmark = gtk_image_new_from_icon_name ("non-starred-symbolic", GTK_ICON_SIZE_MENU);
-  gtk_widget_set_valign (entry->bookmark, GTK_ALIGN_CENTER);
-  gtk_widget_show (entry->bookmark);
-  g_signal_connect (G_OBJECT (entry->bookmark_event_box), "button_press_event", G_CALLBACK (bookmark_icon_button_press_event_cb), entry);
-  gtk_container_add (GTK_CONTAINER (entry->bookmark_event_box), entry->bookmark);
-  gtk_box_pack_end (GTK_BOX (box), entry->bookmark_event_box, FALSE, FALSE, 0);
+  entry->bookmark_button = gtk_button_new_from_icon_name ("non-starred-symbolic", GTK_ICON_SIZE_MENU);
+  gtk_widget_set_tooltip_text (entry->bookmark_button, _("Bookmark this page"));
+  entry->bookmark_icon = gtk_button_get_image (GTK_BUTTON (entry->bookmark_button));
+  gtk_widget_set_valign (entry->bookmark_icon, GTK_ALIGN_CENTER);
+  g_signal_connect (G_OBJECT (entry->bookmark_button), "clicked", G_CALLBACK (bookmark_icon_button_clicked_cb), entry);
+  gtk_box_pack_end (GTK_BOX (box), entry->bookmark_button, FALSE, FALSE, 0);
 
-  context = gtk_widget_get_style_context (entry->bookmark);
+  context = gtk_widget_get_style_context (entry->bookmark_icon);
   gtk_style_context_add_class (context, "entry_icon");
 
   g_settings_bind (EPHY_SETTINGS_LOCKDOWN,
                    EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING,
-                   entry->bookmark_event_box,
+                   entry->bookmark_button,
                    "visible",
                    G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
 
   /* Reader Mode */
-  entry->reader_mode_event_box = gtk_event_box_new ();
-  gtk_widget_set_tooltip_text (entry->reader_mode_event_box, _("Toggle reader mode"));
-  entry->reader_mode = gtk_image_new ();
-  gtk_widget_set_valign (entry->reader_mode, GTK_ALIGN_CENTER);
-  gtk_widget_show (entry->reader_mode);
-  gtk_container_add (GTK_CONTAINER (entry->reader_mode_event_box), entry->reader_mode);
-  gtk_box_pack_end (GTK_BOX (box), entry->reader_mode_event_box, FALSE, FALSE, 0);
+  entry->reader_mode_button = gtk_button_new_from_icon_name (NULL, GTK_ICON_SIZE_MENU);
+  gtk_widget_set_tooltip_text (entry->reader_mode_button, _("Toggle reader mode"));
+  entry->reader_mode_icon = gtk_button_get_image (GTK_BUTTON (entry->reader_mode_button));
+  gtk_widget_set_valign (entry->reader_mode_icon, GTK_ALIGN_CENTER);
+  gtk_box_pack_end (GTK_BOX (box), entry->reader_mode_button, FALSE, FALSE, 0);
 
-  context = gtk_widget_get_style_context (entry->reader_mode);
+  context = gtk_widget_get_style_context (entry->reader_mode_icon);
   gtk_style_context_add_class (context, "entry_icon");
 
   update_reader_icon (entry);
@@ -1085,6 +1151,7 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
                     G_CALLBACK (ephy_location_entry_suggestion_activated), entry);
 
   g_signal_connect (entry->url_entry, "button-release-event", G_CALLBACK (entry_button_release), entry);
+  g_signal_connect (entry->url_entry, "focus-in-event", G_CALLBACK (entry_focus_in_event), entry);
   g_signal_connect (entry->url_entry, "focus-out-event", G_CALLBACK (entry_focus_out_event), entry);
 
   controller = dzl_shortcut_controller_find (entry->url_entry);
@@ -1125,14 +1192,14 @@ ephy_location_entry_new (void)
 }
 
 typedef struct {
-  SoupURI *uri;
+  GUri *uri;
   EphyLocationEntry *entry;
 } PrefetchHelper;
 
 static void
 free_prefetch_helper (PrefetchHelper *helper)
 {
-  soup_uri_free (helper->uri);
+  g_uri_unref (helper->uri);
   g_object_unref (helper->entry);
   g_free (helper);
 }
@@ -1143,7 +1210,7 @@ do_dns_prefetch (PrefetchHelper *helper)
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
 
   if (helper->uri)
-    webkit_web_context_prefetch_dns (ephy_embed_shell_get_web_context (shell), helper->uri->host);
+    webkit_web_context_prefetch_dns (ephy_embed_shell_get_web_context (shell), g_uri_get_host (helper->uri));
 
   helper->entry->dns_prefetch_handle_id = 0;
 
@@ -1191,20 +1258,18 @@ schedule_dns_prefetch (EphyLocationEntry *entry,
 {
   GProxyResolver *resolver = g_proxy_resolver_get_default ();
   PrefetchHelper *helper;
-  SoupURI *uri;
+  g_autoptr (GUri) uri = NULL;
 
   if (resolver == NULL)
     return;
 
-  uri = soup_uri_new (url);
-  if (!uri || !uri->host) {
-    soup_uri_free (uri);
+  uri = g_uri_parse (url, G_URI_FLAGS_NONE, NULL);
+  if (!uri || !g_uri_get_host (uri))
     return;
-  }
 
   helper = g_new0 (PrefetchHelper, 1);
   helper->entry = g_object_ref (entry);
-  helper->uri = uri;
+  helper->uri = g_steal_pointer (&uri);
 
   g_proxy_resolver_lookup_async (resolver, url, NULL, proxy_resolver_ready_cb, helper);
 }
@@ -1320,32 +1385,37 @@ ephy_location_entry_focus (EphyLocationEntry *entry)
 }
 
 void
-ephy_location_entry_set_bookmark_icon_state (EphyLocationEntry                  *entry,
-                                             EphyLocationEntryBookmarkIconState  state)
+ephy_location_entry_set_bookmark_icon_state (EphyLocationEntry     *self,
+                                             EphyBookmarkIconState  state)
 {
   GtkStyleContext *context;
 
-  g_assert (EPHY_IS_LOCATION_ENTRY (entry));
+  self->icon_state = state;
 
-  context = gtk_widget_get_style_context (GTK_WIDGET (entry->bookmark));
+  g_assert (EPHY_IS_LOCATION_ENTRY (self));
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (self->bookmark_icon));
+
+  if (self->adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
+    state = EPHY_BOOKMARK_ICON_HIDDEN;
 
   switch (state) {
-    case EPHY_LOCATION_ENTRY_BOOKMARK_ICON_HIDDEN:
-      gtk_widget_set_visible (entry->bookmark_event_box, FALSE);
+    case EPHY_BOOKMARK_ICON_HIDDEN:
+      gtk_widget_set_visible (self->bookmark_button, FALSE);
       gtk_style_context_remove_class (context, "starred");
       gtk_style_context_remove_class (context, "non-starred");
       break;
-    case EPHY_LOCATION_ENTRY_BOOKMARK_ICON_EMPTY:
-      gtk_widget_set_visible (entry->bookmark_event_box, TRUE);
-      gtk_image_set_from_icon_name (GTK_IMAGE (entry->bookmark),
+    case EPHY_BOOKMARK_ICON_EMPTY:
+      gtk_widget_set_visible (self->bookmark_button, TRUE);
+      gtk_image_set_from_icon_name (GTK_IMAGE (self->bookmark_icon),
                                     "non-starred-symbolic",
                                     GTK_ICON_SIZE_MENU);
       gtk_style_context_remove_class (context, "starred");
       gtk_style_context_add_class (context, "non-starred");
       break;
-    case EPHY_LOCATION_ENTRY_BOOKMARK_ICON_BOOKMARKED:
-      gtk_widget_set_visible (entry->bookmark_event_box, TRUE);
-      gtk_image_set_from_icon_name (GTK_IMAGE (entry->bookmark),
+    case EPHY_BOOKMARK_ICON_BOOKMARKED:
+      gtk_widget_set_visible (self->bookmark_button, TRUE);
+      gtk_image_set_from_icon_name (GTK_IMAGE (self->bookmark_icon),
                                     "starred-symbolic",
                                     GTK_ICON_SIZE_MENU);
       gtk_style_context_remove_class (context, "non-starred");
@@ -1398,20 +1468,20 @@ ephy_location_entry_get_entry (EphyLocationEntry *entry)
 GtkWidget *
 ephy_location_entry_get_bookmark_widget (EphyLocationEntry *entry)
 {
-  return entry->bookmark_event_box;
+  return entry->bookmark_button;
 }
 
 GtkWidget *
 ephy_location_entry_get_reader_mode_widget (EphyLocationEntry *entry)
 {
-  return entry->reader_mode_event_box;
+  return entry->reader_mode_button;
 }
 
 void
 ephy_location_entry_set_reader_mode_visible (EphyLocationEntry *entry,
                                              gboolean           visible)
 {
-  gtk_widget_set_visible (entry->reader_mode_event_box, visible);
+  gtk_widget_set_visible (entry->reader_mode_button, visible);
 }
 
 void
@@ -1419,9 +1489,9 @@ ephy_location_entry_set_reader_mode_state (EphyLocationEntry *entry,
                                            gboolean           active)
 {
   if (active)
-    gtk_style_context_add_class (gtk_widget_get_style_context (entry->reader_mode), "selected");
+    gtk_style_context_add_class (gtk_widget_get_style_context (entry->reader_mode_icon), "selected");
   else
-    gtk_style_context_remove_class (gtk_widget_get_style_context (entry->reader_mode), "selected");
+    gtk_style_context_remove_class (gtk_widget_get_style_context (entry->reader_mode_icon), "selected");
 
   entry->reader_mode_active = active;
 }
@@ -1497,11 +1567,43 @@ ephy_location_entry_set_progress (EphyLocationEntry *entry,
 }
 
 void
-ephy_location_entry_set_mobile_popdown (EphyLocationEntry *entry,
-                                        gboolean           mobile_popdown)
+ephy_location_entry_set_adaptive_mode (EphyLocationEntry *entry,
+                                       EphyAdaptiveMode   adaptive_mode)
 {
-  if (mobile_popdown)
+  if (adaptive_mode == EPHY_ADAPTIVE_MODE_NARROW)
     dzl_suggestion_entry_set_position_func (DZL_SUGGESTION_ENTRY (entry->url_entry), dzl_suggestion_entry_window_position_func, NULL, NULL);
   else
     dzl_suggestion_entry_set_position_func (DZL_SUGGESTION_ENTRY (entry->url_entry), position_func, NULL, NULL);
+
+  entry->adaptive_mode = adaptive_mode;
+
+  update_entry_style (entry);
+
+  ephy_location_entry_set_bookmark_icon_state (entry, entry->icon_state);
+}
+
+void
+ephy_location_entry_page_action_add (EphyLocationEntry *entry,
+                                     GtkWidget         *action)
+{
+  gtk_box_pack_end (GTK_BOX (entry->page_action_box), action, FALSE, FALSE, 0);
+
+  gtk_widget_show (entry->page_action_box);
+}
+
+static void
+clear_page_actions (GtkWidget *child,
+                    gpointer   user_data)
+{
+  EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (user_data);
+
+  gtk_container_remove (GTK_CONTAINER (entry->page_action_box), child);
+}
+
+void
+ephy_location_entry_page_action_clear (EphyLocationEntry *entry)
+{
+  gtk_container_foreach (GTK_CONTAINER (entry->page_action_box), clear_page_actions, entry);
+
+  gtk_widget_hide (entry->page_action_box);
 }
