@@ -49,7 +49,6 @@ struct _EphyActionBarStart {
   GtkWidget *placeholder;
 
   guint navigation_buttons_menu_timeout;
-  gboolean is_fullscreen;
 };
 
 G_DEFINE_TYPE (EphyActionBarStart, ephy_action_bar_start, GTK_TYPE_BOX)
@@ -106,23 +105,14 @@ icon_loaded_cb (GObject      *source,
   cairo_surface_t *icon_surface = webkit_favicon_database_get_favicon_finish (database, result, NULL);
 
   if (icon_surface) {
-    favicon = ephy_pixbuf_get_from_surface_scaled (icon_surface, FAVICON_SIZE, FAVICON_SIZE);
+    gint scale = gtk_widget_get_scale_factor (image);
+
+    favicon = ephy_pixbuf_get_from_surface_scaled (icon_surface, FAVICON_SIZE * scale, FAVICON_SIZE * scale);
     cairo_surface_destroy (icon_surface);
   }
 
-  if (favicon) {
-    cairo_surface_t *surface;
-
-    surface = gdk_cairo_surface_create_from_pixbuf (favicon,
-                                                    0,
-                                                    gtk_widget_get_window (GTK_WIDGET (image)));
-    gtk_image_set_from_surface (GTK_IMAGE (image), surface);
-
-    gtk_widget_show (image);
-
-    cairo_surface_destroy (surface);
-    g_object_unref (favicon);
-  }
+  if (favicon)
+    gtk_image_set_from_gicon (GTK_IMAGE (image), G_ICON (favicon), GTK_ICON_SIZE_MENU);
 
   g_object_unref (image);
 }
@@ -392,6 +382,8 @@ navigation_button_release_event_cb (GtkButton *button,
                                            "navigation-forward-new-tab");
       g_action_activate (action, NULL);
     }
+
+    /* Don't propagate the event to avoid other middle-click actions. */
     return GDK_EVENT_STOP;
   }
 
@@ -405,9 +397,13 @@ navigation_button_release_event_cb (GtkButton *button,
                                            "navigation-forward");
       g_action_activate (action, NULL);
     }
-    return GDK_EVENT_STOP;
+
+    /* Propagate the event to allow the button to correctly reset its internal
+     * state. */
+    return GDK_EVENT_PROPAGATE;
   }
 
+  /* Propagate other unhandled events. */
   return GDK_EVENT_PROPAGATE;
 }
 
@@ -506,8 +502,7 @@ update_new_tab_button_visibility (EphyActionBarStart *action_bar_start)
 
   gtk_widget_set_visible (action_bar_start->new_tab_button,
                           (ephy_embed_shell_get_mode (embed_shell) != EPHY_EMBED_SHELL_MODE_APPLICATION) &&
-                          !is_desktop_pantheon () &&
-                          !action_bar_start->is_fullscreen);
+                          !is_desktop_pantheon ());
 }
 
 static void
@@ -575,7 +570,7 @@ ephy_action_bar_start_constructed (GObject *object)
   if (ephy_profile_dir_is_web_application ()) {
     GtkWidget *navigation_box = ephy_action_bar_start_get_navigation_box (action_bar_start);
 
-    g_settings_bind (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_MOBILE_CAPABLE, navigation_box, "visible", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
+    g_settings_bind (EPHY_SETTINGS_WEB_APP, EPHY_PREFS_WEB_APP_SHOW_NAVIGATION_BUTTONS, navigation_box, "visible", G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
   }
 }
 
@@ -620,7 +615,6 @@ ephy_action_bar_start_class_init (EphyActionBarStartClass *klass)
 static void
 ephy_action_bar_start_init (EphyActionBarStart *action_bar_start)
 {
-  action_bar_start->is_fullscreen = FALSE;
 }
 
 EphyActionBarStart *
@@ -656,17 +650,6 @@ ephy_action_bar_start_change_combined_stop_reload_state (EphyActionBarStart *act
   }
 }
 
-void
-ephy_action_bar_set_is_fullscreen (EphyActionBarStart *action_bar_start,
-                                   gboolean            fullscreen)
-{
-  if (action_bar_start->is_fullscreen == fullscreen)
-    return;
-
-  action_bar_start->is_fullscreen = fullscreen;
-  update_new_tab_button_visibility (action_bar_start);
-}
-
 GtkWidget *
 ephy_action_bar_start_get_placeholder (EphyActionBarStart *action_bar_start)
 {
